@@ -4,20 +4,21 @@ package JSON::Schema::Modern::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.524';
+our $VERSION = '0.525';
 
-use 5.016;
+use 5.020;
+use Moo;
+use strictures 2;
+use experimental qw(signatures postderef);
+use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, experimental => 're_strict';
-use strictures 2;
 use Mojo::URL;
 use Carp 'croak';
 use List::Util 1.29 'pairs';
 use Ref::Util 0.100 'is_plain_hashref';
 use Safe::Isa;
-use Moo;
 use MooX::TypeTiny;
 use MooX::HandlesVia;
 use Types::Standard qw(InstanceOf HashRef Str Dict ArrayRef Enum ClassName Undef slurpy);
@@ -144,9 +145,7 @@ sub FOREIGNBUILDARGS { () }
 # for JSON serializers
 sub TO_JSON { goto \&schema }
 
-sub BUILD {
-  my ($self, $args) = @_;
-
+sub BUILD ($self, $args) {
   croak 'canonical_uri cannot contain a fragment' if defined $self->canonical_uri->fragment;
 
   my $original_uri = $self->canonical_uri->clone;
@@ -155,7 +154,7 @@ sub BUILD {
   # if the schema identified a canonical uri for itself, it overrides the initial value
   $self->_set_canonical_uri($state->{initial_schema_uri}) if $state->{initial_schema_uri} ne $original_uri;
 
-  if (@{$state->{errors}}) {
+  if ($state->{errors}->@*) {
     $self->_set_errors($state->{errors});
     return;
   }
@@ -170,15 +169,13 @@ sub BUILD {
     if (not "$original_uri" and $original_uri eq $self->canonical_uri)
       or "$original_uri";
 
-  $self->_add_resources(@{$state->{identifiers}});
+  $self->_add_resources($state->{identifiers}->@*);
 
   # overlay the resulting configs with those that were provided by the caller
-  $self->_set_evaluation_configs(+{ %{$state->{configs}}, %{$self->evaluation_configs} });
+  $self->_set_evaluation_configs(+{ $state->{configs}->%*, $self->evaluation_configs->%* });
 }
 
-sub traverse {
-  my ($self, $evaluator) = @_;
-
+sub traverse ($self, $evaluator) {
   my $state = $evaluator->traverse($self->schema,
     {
       initial_schema_uri => $self->canonical_uri->clone,
@@ -196,9 +193,7 @@ sub traverse {
   return $state;
 }
 
-sub validate {
-  my $self = shift;
-
+sub validate ($self) {
   my $js = $self->$_call_if_can('evaluator') // JSON::Schema::Modern->new;
 
   return $js->evaluate($self->schema, $self->metaschema_uri);
@@ -220,7 +215,7 @@ JSON::Schema::Modern::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.524
+version 0.525
 
 =head1 SYNOPSIS
 
