@@ -4,20 +4,24 @@ package JSON::Schema::Modern::Error;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single error from a JSON Schema evaluation
 
-our $VERSION = '0.524';
+our $VERSION = '0.525';
 
-use 5.016;
+use 5.020;
+use Moo;
+use strictures 2;
+use experimental qw(signatures postderef);
+use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, experimental => 're_strict';
-use strictures 2;
 use Safe::Isa;
-use Moo;
 use MooX::TypeTiny;
 use Types::Standard qw(Str Undef InstanceOf);
 use namespace::clean;
 
+use overload
+  '""' => sub { $_[0]->stringify };
+
 has [qw(
   instance_location
   keyword_location
@@ -40,8 +44,7 @@ has keyword => (
   required => 1,
 );
 
-sub TO_JSON {
-  my $self = shift;
+sub TO_JSON ($self) {
   return +{
     # note that locations are JSON pointers, not uri fragments!
     instanceLocation => $self->instance_location,
@@ -52,6 +55,10 @@ sub TO_JSON {
   };
 }
 
+sub stringify ($self) {
+    $self->keyword_location.': '.$self->error."\n";
+}
+
 1;
 
 __END__
@@ -68,7 +75,7 @@ JSON::Schema::Modern::Error - Contains a single error from a JSON Schema evaluat
 
 =head1 VERSION
 
-version 0.524
+version 0.525
 
 =head1 SYNOPSIS
 
@@ -117,6 +124,8 @@ The actual error string.
 
 =head1 METHODS
 
+=for Pod::Coverage stringify
+
 =head2 TO_JSON
 
 Returns a data structure suitable for serialization. Corresponds to one output unit as specified in
