@@ -4,18 +4,19 @@ package JSON::Schema::Modern::Vocabulary::Validation;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Validation vocabulary
 
-our $VERSION = '0.524';
+our $VERSION = '0.525';
 
-use 5.016;
+use 5.020;
+use Moo;
+use strictures 2;
+use experimental qw(signatures postderef);
+use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use if "$]" >= 5.022, experimental => 're_strict';
-use strictures 2;
 use List::Util 'any';
 use Ref::Util 0.100 'is_plain_arrayref';
 use JSON::Schema::Modern::Utilities qw(is_type is_equal is_elements_unique E assert_keyword_type assert_pattern jsonp);
-use Moo;
 use namespace::clean;
 
 with 'JSON::Schema::Modern::Vocabulary';
@@ -27,8 +28,7 @@ sub vocabulary {
 
 sub evaluation_order { 2 }
 
-sub keywords {
-  my ($self, $spec_version) = @_;
+sub keywords ($self, $spec_version) {
   return (
     qw(type enum const
       multipleOf maximum exclusiveMaximum minimum exclusiveMinimum
@@ -40,12 +40,10 @@ sub keywords {
   );
 }
 
-sub _traverse_keyword_type {
-  my ($self, $schema, $state) = @_;
-
+sub _traverse_keyword_type ($self, $schema, $state) {
   if (is_plain_arrayref($schema->{type})) {
-    return E($state, 'type array is empty') if not @{$schema->{type}};
-    foreach my $type (@{$schema->{type}}) {
+    return E($state, 'type array is empty') if not $schema->{type}->@*;
+    foreach my $type ($schema->{type}->@*) {
       return E($state, 'unrecognized type "%s"', $type//'<null>')
         if not any { ($type//'') eq $_ } qw(null boolean object array string number integer);
     }
@@ -59,16 +57,14 @@ sub _traverse_keyword_type {
   return 1;
 }
 
-sub _eval_keyword_type {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_type ($self, $data, $schema, $state) {
   if (is_plain_arrayref($schema->{type})) {
-    # return 1 if any { is_type($_, $data) } @{$schema->{type}};
-    foreach my $type (@{$schema->{type}}) {
+    # return 1 if any { is_type($_, $data) } $schema->{type}->@*;
+    foreach my $type ($schema->{type}->@*) {
       return 1 if is_type($type, $data)
         or ($type eq 'boolean' and $state->{scalarref_booleans} and is_type('reference to SCALAR', $data));
     }
-    return E($state, 'wrong type (expected one of %s)', join(', ', @{$schema->{type}}));
+    return E($state, 'wrong type (expected one of %s)', join(', ', $schema->{type}->@*));
   }
   else {
     return 1 if is_type($schema->{type}, $data)
@@ -77,19 +73,16 @@ sub _eval_keyword_type {
   }
 }
 
-sub _traverse_keyword_enum {
-  my ($self, $schema, $state) = @_;
+sub _traverse_keyword_enum ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'array');
   return E($state, '"enum" values are not unique') if not is_elements_unique($schema->{enum});
   return 1;
 }
 
-sub _eval_keyword_enum {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_enum ($self, $data, $schema, $state) {
   my @s; my $idx = 0;
   my %s = ( scalarref_booleans => $state->{scalarref_booleans} );
-  return 1 if any { is_equal($data, $_, $s[$idx++] = {%s}) } @{$schema->{enum}};
+  return 1 if any { is_equal($data, $_, $s[$idx++] = {%s}) } $schema->{enum}->@*;
 
   return E($state, 'value does not match'
     .(!(grep $_->{path}, @s) ? ''
@@ -98,25 +91,20 @@ sub _eval_keyword_enum {
 
 sub _traverse_keyword_const { 1 }
 
-sub _eval_keyword_const {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_const ($self, $data, $schema, $state) {
   my %s = ( scalarref_booleans => $state->{scalarref_booleans} );
   return 1 if is_equal($data, $schema->{const}, my $s = { scalarref_booleans => $state->{scalarref_booleans} });
   return E($state, 'value does not match'
     .($s->{path} ? ' (differences start at "'.$s->{path}.'")' : ''));
 }
 
-sub _traverse_keyword_multipleOf {
-  my ($self, $schema, $state) = @_;
+sub _traverse_keyword_multipleOf ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'number');
   return E($state, 'multipleOf value is not a positive number') if $schema->{multipleOf} <= 0;
   return 1;
 }
 
-sub _eval_keyword_multipleOf {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_multipleOf ($self, $data, $schema, $state) {
   return 1 if not is_type('number', $data);
 
   my $quotient = $data / $schema->{multipleOf};
@@ -126,9 +114,7 @@ sub _eval_keyword_multipleOf {
 
 sub _traverse_keyword_maximum { goto \&_assert_number }
 
-sub _eval_keyword_maximum {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_maximum ($self, $data, $schema, $state) {
   return 1 if not is_type('number', $data);
   return 1 if $data <= $schema->{maximum};
   return E($state, 'value is larger than %g', $schema->{maximum});
@@ -136,9 +122,7 @@ sub _eval_keyword_maximum {
 
 sub _traverse_keyword_exclusiveMaximum { goto \&_assert_number }
 
-sub _eval_keyword_exclusiveMaximum {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_exclusiveMaximum ($self, $data, $schema, $state) {
   return 1 if not is_type('number', $data);
   return 1 if $data < $schema->{exclusiveMaximum};
   return E($state, 'value is equal to or larger than %g', $schema->{exclusiveMaximum});
@@ -146,9 +130,7 @@ sub _eval_keyword_exclusiveMaximum {
 
 sub _traverse_keyword_minimum { goto \&_assert_number }
 
-sub _eval_keyword_minimum {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_minimum ($self, $data, $schema, $state) {
   return 1 if not is_type('number', $data);
   return 1 if $data >= $schema->{minimum};
   return E($state, 'value is smaller than %g', $schema->{minimum});
@@ -156,9 +138,7 @@ sub _eval_keyword_minimum {
 
 sub _traverse_keyword_exclusiveMinimum { goto \&_assert_number }
 
-sub _eval_keyword_exclusiveMinimum {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_exclusiveMinimum ($self, $data, $schema, $state) {
   return 1 if not is_type('number', $data);
   return 1 if $data > $schema->{exclusiveMinimum};
   return E($state, 'value is equal to or smaller than %g', $schema->{exclusiveMinimum});
@@ -166,9 +146,7 @@ sub _eval_keyword_exclusiveMinimum {
 
 sub _traverse_keyword_maxLength { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_maxLength {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_maxLength ($self, $data, $schema, $state) {
   return 1 if not is_type('string', $data);
   return 1 if length($data) <= $schema->{maxLength};
   return E($state, 'length is greater than %d', $schema->{maxLength});
@@ -176,24 +154,20 @@ sub _eval_keyword_maxLength {
 
 sub _traverse_keyword_minLength { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_minLength {
-  my ($self, $data, $schema, $state) = @_;
+sub _eval_keyword_minLength ($self, $data, $schema, $state) {
 
   return 1 if not is_type('string', $data);
   return 1 if length($data) >= $schema->{minLength};
   return E($state, 'length is less than %d', $schema->{minLength});
 }
 
-sub _traverse_keyword_pattern {
-  my ($self, $schema, $state) = @_;
+sub _traverse_keyword_pattern ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'string')
     or not assert_pattern($state, $schema->{pattern});
   return 1;
 }
 
-sub _eval_keyword_pattern {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_pattern ($self, $data, $schema, $state) {
   return 1 if not is_type('string', $data);
 
   return 1 if $data =~ m/$schema->{pattern}/;
@@ -202,9 +176,7 @@ sub _eval_keyword_pattern {
 
 sub _traverse_keyword_maxItems { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_maxItems {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_maxItems ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if @$data <= $schema->{maxItems};
   return E($state, 'more than %d item%s', $schema->{maxItems}, $schema->{maxItems} > 1 ? 's' : '');
@@ -212,23 +184,18 @@ sub _eval_keyword_maxItems {
 
 sub _traverse_keyword_minItems { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_minItems {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_minItems ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if @$data >= $schema->{minItems};
   return E($state, 'fewer than %d item%s', $schema->{minItems}, $schema->{minItems} > 1 ? 's' : '');
 }
 
-sub _traverse_keyword_uniqueItems {
-  my ($self, $schema, $state) = @_;
+sub _traverse_keyword_uniqueItems ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'boolean');
   return 1;
 }
 
-sub _eval_keyword_uniqueItems {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_uniqueItems ($self, $data, $schema, $state) {
   return 1 if not is_type('array', $data);
   return 1 if not $schema->{uniqueItems};
   return 1 if is_elements_unique($data, my $equal_indices = []);
@@ -238,9 +205,7 @@ sub _eval_keyword_uniqueItems {
 # Note: no effort is made to check if the 'contains' keyword has been disabled via its vocabulary.
 sub _traverse_keyword_maxContains { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_maxContains {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_maxContains ($self, $data, $schema, $state) {
   return 1 if not exists $state->{_num_contains};
   return 1 if not is_type('array', $data);
 
@@ -252,9 +217,7 @@ sub _eval_keyword_maxContains {
 
 sub _traverse_keyword_minContains { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_minContains {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_minContains ($self, $data, $schema, $state) {
   return 1 if not exists $state->{_num_contains};
   return 1 if not is_type('array', $data);
 
@@ -266,9 +229,7 @@ sub _eval_keyword_minContains {
 
 sub _traverse_keyword_maxProperties { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_maxProperties {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_maxProperties ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
   return 1 if keys %$data <= $schema->{maxProperties};
   return E($state, 'more than %d propert%s', $schema->{maxProperties},
@@ -277,46 +238,38 @@ sub _eval_keyword_maxProperties {
 
 sub _traverse_keyword_minProperties { goto \&_assert_non_negative_integer }
 
-sub _eval_keyword_minProperties {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_minProperties ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
   return 1 if keys %$data >= $schema->{minProperties};
   return E($state, 'fewer than %d propert%s', $schema->{minProperties},
     $schema->{minProperties} > 1 ? 'ies' : 'y');
 }
 
-sub _traverse_keyword_required {
-  my ($self, $schema, $state) = @_;
-
+sub _traverse_keyword_required ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'array');
   return E($state, '"required" element is not a string')
-    if any { !is_type('string', $_) } @{$schema->{required}};
+    if any { !is_type('string', $_) } $schema->{required}->@*;
   return E($state, '"required" values are not unique') if not is_elements_unique($schema->{required});
   return 1;
 }
 
-sub _eval_keyword_required {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_required ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
-  my @missing = grep !exists $data->{$_}, @{$schema->{required}};
+  my @missing = grep !exists $data->{$_}, $schema->{required}->@*;
   return 1 if not @missing;
   return E($state, 'missing propert%s: %s', @missing > 1 ? 'ies' : 'y', join(', ', @missing));
 }
 
-sub _traverse_keyword_dependentRequired {
-  my ($self, $schema, $state) = @_;
-
+sub _traverse_keyword_dependentRequired ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'object');
 
   my $valid = 1;
-  foreach my $property (sort keys %{$schema->{dependentRequired}}) {
+  foreach my $property (sort keys $schema->{dependentRequired}->%*) {
     $valid = E({ %$state, _schema_path_suffix => $property }, 'value is not an array'), next
       if not is_type('array', $schema->{dependentRequired}{$property});
 
-    foreach my $index (0..$#{$schema->{dependentRequired}{$property}}) {
+    foreach my $index (0..$schema->{dependentRequired}{$property}->$#*) {
       $valid = E({ %$state, _schema_path_suffix => [ $property, $index ] }, 'element #%d is not a string', $index)
         if not is_type('string', $schema->{dependentRequired}{$property}[$index]);
     }
@@ -327,16 +280,14 @@ sub _traverse_keyword_dependentRequired {
   return $valid;
 }
 
-sub _eval_keyword_dependentRequired {
-  my ($self, $data, $schema, $state) = @_;
-
+sub _eval_keyword_dependentRequired ($self, $data, $schema, $state) {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  foreach my $property (sort keys %{$schema->{dependentRequired}}) {
+  foreach my $property (sort keys $schema->{dependentRequired}->%*) {
     next if not exists $data->{$property};
 
-    if (my @missing = grep !exists($data->{$_}), @{$schema->{dependentRequired}{$property}}) {
+    if (my @missing = grep !exists($data->{$_}), $schema->{dependentRequired}{$property}->@*) {
       $valid = E({ %$state, _schema_path_suffix => $property },
         'missing propert%s: %s', @missing > 1 ? 'ies' : 'y', join(', ', @missing));
     }
@@ -346,14 +297,12 @@ sub _eval_keyword_dependentRequired {
   return E($state, 'not all dependencies are satisfied');
 }
 
-sub _assert_number {
-  my ($self, $schema, $state) = @_;
+sub _assert_number ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'number');
   return 1;
 }
 
-sub _assert_non_negative_integer {
-  my ($self, $schema, $state) = @_;
+sub _assert_non_negative_integer ($self, $schema, $state) {
   return if not assert_keyword_type($state, $schema, 'integer');
   return E($state, '%s value is not a non-negative integer', $state->{keyword})
     if $schema->{$state->{keyword}} < 0;
@@ -374,7 +323,7 @@ JSON::Schema::Modern::Vocabulary::Validation - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.524
+version 0.525
 
 =head1 DESCRIPTION
 
